﻿//
//  JockeyJS
//
//  Copyright (c) 2013, Tim Coulter
//
//  Permission is hereby granted, free of charge, to any person obtaining
//  a copy of this software and associated documentation files (the
//  "Software"), to deal in the Software without restriction, including
//  without limitation the rights to use, copy, modify, merge, publish,
//  distribute, sublicense, and/or sell copies of the Software, and to
//  permit persons to whom the Software is furnished to do so, subject to
//  the following conditions:
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
//  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
//  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
//  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

; (function () {

    // Non-accessible variable to send to the app, to ensure events only
    // come from the desired host.
    var host = window.location.host;

    var Dispatcher = {
        callbacks: {},

        send: function (envelope, complete) {
            this.dispatchMessage("event", envelope, complete);
        },

        sendCallback: function (messageId, payload) {
            var envelope = Jockey.createEnvelope(messageId, "", payload);

            this.dispatchMessage("callback", envelope, function () { });
        },

        triggerCallback: function (id, payload) {
            var dispatcher = this;

            // Alerts within JS callbacks will sometimes freeze the iOS app.
            // Let's wrap the callback in a timeout to prevent this.
            setTimeout(function () {
                dispatcher.callbacks[id](payload);
            }, 0);
        },

        // `type` can either be "event" or "callback"
        dispatchMessage: function (type, envelope, complete) {
            // We send the message by navigating the browser to a special URL.
            // The iOS library will catch the navigation, prevent the UIWebView
            // from continuing, and use the data in the URL to execute code
            // within the iOS app.

            var dispatcher = this;

            this.callbacks[envelope.id] = function (payload) {
                complete(payload);

                delete dispatcher.callbacks[envelope.id];
            };

            var src = "jockey://" + type + "/" + envelope.id + "?" + encodeURIComponent(JSON.stringify(envelope));
            var iframe = document.createElement("iframe");
            iframe.setAttribute("src", src);
            document.documentElement.appendChild(iframe);
            iframe.parentNode.removeChild(iframe);
            iframe = null;
        }
    };

    var Jockey = {
        listeners: {},

        dispatcher: null,

        messageCount: 0,

        on: function (type, fn) {
            if (!this.listeners.hasOwnProperty(type) || !this.listeners[type] instanceof Array) {
                this.listeners[type] = [];
            }

            this.listeners[type].push(fn);
        },

        off: function (type) {
            if (!this.listeners.hasOwnProperty(type) || !this.listeners[type] instanceof Array) {
                this.listeners[type] = [];
            }

            this.listeners[type] = [];
        },

        send: function (type, payload, complete) {
            if (payload instanceof Function) {
                complete = payload;
                payload = null;
            }

            payload = payload || {};
            complete = complete || function () { };

            var envelope = this.createEnvelope(this.messageCount, type, payload);

            this.dispatcher.send(envelope, complete);

            this.messageCount += 1;
        },

        // Called by the native application when events are sent to JS from the app.
        // Will execute every function, FIFO order, that was attached to this event type.
        trigger: function (type, messageId, json) {
            var self = this;

            var listenerList = this.listeners[type] || [];

            var executedCount = 0;

            var complete = function (response) {
                executedCount += 1;

                if (executedCount >= listenerList.length) {
                    self.dispatcher.sendCallback(messageId, response);
                }
            };

            for (var index = 0; index < listenerList.length; index++) {
                var listener = listenerList[index];

                // If it's a "sync" listener, we'll call the complete() function
                // after it has finished. If it's async, we expect it to call complete().
                if (listener.length <= 1) {
                    listener(json);
                    complete(json);
                } else {
                    listener(json, complete);
                }
            }

        },

        // Called by the native application in response to an event sent to it.
        // This will trigger the callback passed to the send() function for
        // a given message.
        triggerCallback: function (id, payload) {
            this.dispatcher.triggerCallback(id, payload);
        },

        createEnvelope: function (id, type, payload) {
            return {
                id: id,
                type: type,
                host: host,
                payload: payload
            };
        }
    };

    // i.e., on a Desktop browser.
    var nullDispatcher = {
        send: function (envelope, complete) { /*complete();*/ },
        triggerCallback: function () { },
        sendCallback: function () { }
    };

    // Dispatcher detection. Currently only supports iOS.
    // Looking for equivalent Android implementation.
    var i = 0,
        iOS = false,
        iDevice = ['iPad', 'iPhone', 'iPod'];

    for (; i < iDevice.length; i++) {
        if (navigator.platform.indexOf(iDevice[i]) >= 0) {
            iOS = true;
            break;
        }
    }

    // Detect UIWebview. In Mobile Safari proper, jockey urls cause a popup to
    // be shown that says "Safari cannot open page because the URL is invalid."
    // From here: http://stackoverflow.com/questions/4460205/detect-ipad-iphone-webview-via-javascript

    var UIWebView = /(iPhone|iPod|iPad).*AppleWebKit(?!.*Safari)/i.test(navigator.userAgent);
    var isAndroid = navigator.userAgent.toLowerCase().indexOf("android") > -1;

    if ((iOS && UIWebView) || isAndroid) {
        Jockey.dispatcher = Dispatcher;
    } else {
        Jockey.dispatcher = nullDispatcher;
    }

    window.Jockey = Jockey;
})();

; (function (window, factory) {

    if (typeof define === 'function' && define.amd) {
        //define(factory);
        define(['jweixin', 'jockey'], factory);//TODO 仅在微信或原生端才有必要加载这两个插件
    } else if (typeof exports === 'object') {
        module.exports = factory(); //TODO module.exports = factory(require('jweixin','jockey'));
    } else {
        window.wx = window.wx || null;
        window.HybridBridge = factory(wx, Jockey);
    }
} (this, function (wx, Jockey) {
    var os = {
        Android: function () {
            return navigator.userAgent.match(/Android/i) != null;
        },
        iOS: function () {
            return navigator.userAgent.match(/iPhone|iPad|iPod/i) != null;
        },
        WeiChat: function () {
            return navigator.userAgent.match(/micromessenger/i) != null;
        },
        OurApp: function () {
            return navigator.userAgent.match(/dkhs_/i) != null;
        },
        QQBrowser: function () {
            return navigator.userAgent.match(/QQ/i) != null;
        },
        Mobile: function () {
            return (os.Android() || os.iOS() || os.OurApp() || os.WeiChat());
        }
    },
        apiEvents = {
            config: 'config',
            checkJsApi: 'checkJsApi',

            onMenuShareTimeline: 'onMenuShareTimeline',
            shareTimeline: 'shareTimeline',
            onMenuShareAppMessage: 'onMenuShareAppMessage',
            shareAppMessage: 'shareAppMessage',
            onMenuShareQQ: 'onMenuShareQQ',
            shareQQ: 'shareQQ',
            onMenuShareWeibo: 'onMenuShareWeibo',
            shareWeibo: 'shareWeibo',
            onMenuShareQZone: 'onMenuShareQZone',
            shareQZone: 'shareQZone',

            hideMenuItems: 'hideMenuItems',
            showMenuItems: 'showMenuItems',
            hideAllNonBaseMenuItem: 'hideAllNonBaseMenuItem',
            showAllNonBaseMenuItem: 'showAllNonBaseMenuItem',

            hideOptionMenu: 'hideOptionMenu',
            showOptionMenu: 'showOptionMenu',
            closeWindow: 'closeWindow',

            resizeWindow: 'resizeWindow',
            onDownloadImage: 'downloadImage',
            adjustFontSize: 'adjustFontSize',
            openImageGalley: 'openImageGalley',
            loadedAllImage:'loadedAllImage',
            openMiniProgram:'openMiniProgram'
        };



    //function bridgeCall(bridgeToNative,bridgeToWechat) {
    //    if (os.OurApp() && bridgeToNative && typeof bridgeToNative === 'function') {
    //        bridgeToNative();
    //    }
    //    else if (os.WeiChat() && bridgeToWechat && typeof bridgeToWechat === 'function') {
    //        bridgeToWechat();
    //    }
    //}

    /**@description 判断当前的客户端是否支持 该API
     * @options {version} 版本号
     * @options {apiName} api名称
     * @return {true/false}
     */
    var isApiSupport = function (version, apiName) {
        //判断某个API，自某个版本号就开始支持
        return true;
    }

    /**@description 获取当前客户端的版本号
    * @return {string 2.7.0}
    */
    var getNativeHybridVersion = function () {
        return {
            platform: '',
            version: ''
        };
        //return version;
    };


    function shareInfoFormat(options) {
        options = options || {};
        return {
            title: options.title || document.title,
            desc: options.desc || '',
            imgUrl: options.imgUrl || "",
            link: options.link || location.href,
            type: options.type || "link",
            dataUrl: options.dataUrl || "",
            success: options.success || function () { },
            cancel: options.cancel || function () { }
        };
    }

    function configShare(options, wxOnMenuShareEvent, ourOnMenuShareEvent, ourShareEvent) {
        if (os.WeiChat()) {
            //设置微信端的分享内容
            wx.ready(function () {
                wxOnMenuShareEvent(options);
            });
        }
        else if (os.OurApp()) {
            //监听客户端的分享点击事件，然后向客户端发送要分享的内容
            Jockey.on(ourOnMenuShareEvent, function (payload, complete) {
                Jockey.send(ourShareEvent, shareInfoFormat(options), function (res) {
                    switch (res.status) {
                        case 'success':
                            options.success && options.success(res);
                            break;
                        case 'fail':
                            options.fail && options.fail(res);
                            break;
                        case 'cancel':
                            options.cancel && options.cancel(res);
                            break;
                    }
                    options.complete && options.complete(res);

                });
            });
        }
    }

    var HybridBridge = {
        /**@description 微信配置
         * @options {options} 配置内容appId，timestamp，nonceStr，signature等
         * @return {void}
         */
        os: os,
        debug: false,
        config: function (options) {
            options = options || {};
            if (os.WeiChat()) {
                wx.config(options);
            } else if (os.OurApp()) {
                Jockey.send(apiEvents.config, options, function (res) {
                });
            }
            this.debug = options.debug || false;
        },
        /** @description 监听点击分享到朋友圈按钮时事件
         * @param {options} 把要分享的内容告诉给客户端,options里包含success/fail/cancel/complete回调
         * @return {void}
         */
        onMenuShareTimeline: function (options) {
            configShare(options, wx.onMenuShareTimeline, apiEvents.onMenuShareTimeline, apiEvents.shareTimeline);

        },
        /** @description 监听点击分享到朋友按钮时事件
         * @param {options} 把要分享的内容告诉给客户端,options里包含success/fail/cancel/complete回调
         * @return {void}
         */
        onMenuShareAppMessage: function (options) {
            configShare(options, wx.onMenuShareAppMessage, apiEvents.onMenuShareAppMessage, apiEvents.shareAppMessage);
        },
        /** @description 监听点击分享到朋友按钮时事件
         * @param {options} 把要分享的内容告诉给客户端,options里包含success/fail/cancel/complete回调
         * @return {void}
         */
        onMenuShareQQ: function (options) {
            configShare(options, wx.onMenuShareQQ, apiEvents.onMenuShareQQ, apiEvents.shareQQ);
        },
        /** @description 监听点击分享到朋友按钮时事件
         * @param {options} 把要分享的内容告诉给客户端,options里包含success/fail/cancel/complete回调
         * @return {void}
         */
        onMenuShareWeibo: function (options) {
            configShare(options, wx.onMenuShareWeibo, apiEvents.onMenuShareWeibo, apiEvents.shareWeibo);
        },
        /** @description 监听点击分享到朋友按钮时事件
         * @param {options} 把要分享的内容告诉给客户端,options里包含success/fail/cancel/complete回调
         * @return {void}
         */
        onMenuShareQZone: function (options) {
            configShare(options, wx.onMenuShareQZone, apiEvents.onMenuShareQZone, apiEvents.shareQZone);
        },
        /** @description 统一监听所有分享按钮点击事件
         * @param {options} 把要分享的内容告诉给客户端,options里包含success/fail/cancel/complete回调
         * @return {void}
         */
        onMenuShareAll: function (options) {
            this.onMenuShareTimeline(options);
            this.onMenuShareAppMessage(options);
            this.onMenuShareQQ(options);
            this.onMenuShareWeibo(options);
            this.onMenuShareQZone(options);
        },
        /**@description 隐藏微信右上角按钮或客户端右上角的按钮
         * @param {无}
         * @return {无}
         */
        hideOptionMenu: function () {
            if (os.WeiChat()) {
                wx.hideOptionMenu();
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.hideOptionMenu, {}, function (res) { });
            }
        },
        /**@description 显示微信右上角按钮或客户端右上角的按钮
         * @param {无}
         * @return {无}
         */
        showOptionMenu: function () {
            if (os.WeiChat()) {
                wx.showOptionMenu();
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.showOptionMenu, {}, function (res) { });
            }
        },
        /**@description 告诉原生APP，关闭当前webview，如果有接收到reload参数为true,则要更新最后打开的那个APP页面
          * @param {reload:true/false}
          * @return {无}
          */
        closeWindow: function (options) {
            if (os.WeiChat()) {
                wx.closeWindow();
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.closeWindow, options, function (res) { });
            }
        },
        /**@description 告诉native,当前webview的高度有发生了变化
          * @param {无}
          * @return {无}
          */
        resizeWindow: function () {
            if (os.WeiChat()) {
                //微信不支持该API
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.resizeWindow, {}, function (res) { });
            }
        },
        /**@description 告诉web,客户端已经下载好一远程图片，然后返回一个本地图片路径
         * @param {complete:function() }
        * @return {index:0,localUrl:""}
        */
        onDownloadImage: function (options) {
            if (os.WeiChat()) {
                //微信不支持该API
            }
            else if (os.OurApp()) {
                Jockey.on(apiEvents.onDownloadImage, function (payload, complete) {
                    options.complete && options.complete(payload);
                });

            }
        },
        adjustFontSize: function (options) {
                    if (os.WeiChat()) {
                        //微信不支持该API
                    }
                    else if (os.OurApp()) {
                        Jockey.on(apiEvents.adjustFontSize, function (size, complete) {
                            options.complete && options.complete(size);
                        });

                    }
                },
        /**@description 告诉native,我要打开哪个图片，和所有图片的位置
          * @param {index:'',images:[[x,y,w,h],[x,y,w,h],[x,y,w,h],[x,y,w,h]}
          * @return {index:""}
          */
        openImageGalley: function (options) {
            if (os.WeiChat()) {
                //微信不支持该API
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.openImageGalley, options, function (res) {
                    options.complete && options.complete(res);
                });
            }
        },
        /**@description 告诉native,我的所有图片DOM已经加载好了，并且每张图片所在的绝对位置
          * @param {images:[[x,y,w,h],[x,y,w,h],[x,y,w,h],[x,y,w,h]}
          * @return {index:""}
          */
        loadedAllImage: function (options) {
            if (os.WeiChat()) {
                //微信不支持该API
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.loadedAllImage, options, function (res) {
                    options.complete && options.complete(res);
                });
            }
        },
        /**@description 告诉native,我想绑定微信第三方帐号
          * @param
          * @return true/false
          */
        bindWeixin: function (options) {
              if (os.WeiChat()) {
                   //微信不支持该API
              }
              else if (os.OurApp()) {
                  Jockey.send(apiEvents.bindWeixin, options, function (res) {
                      options.complete && options.complete(res);
                  });
              }
         },

       /**@description 告诉native,我想弹出底部的第三方分享按钮
       * @param
       * @return true/false
       */
        openShareMenu: function (options) {
            if (os.WeiChat()) {
                //微信不支持该API
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.openShareMenu, options, function (res) {
                    options.complete && options.complete(res);
                });
            }
        },

        /**@description 告诉native,我想在新的页面中打开一个页面,传1个参数， url
         * @param {url:''}
         * @return true/false
         */
        openWebview: function (options) {
            if (options.url) {
                options.url = location.protocol + '//' + location.host + options.url;
            }
            if (os.WeiChat()) {
                //微信不支持该API
            }
            else if (os.OurApp()) {
                Jockey.send(apiEvents.openWebview, options, function (res) {
                    options.complete && options.complete(res);
                });
            }
        },

         openMiniProgram: function (options) {
            if (os.OurApp()) {
                Jockey.send(apiEvents.openMiniProgram, options, function (res) {
                    options.complete && options.complete(res);
                });
            }
         },

    }

    return HybridBridge;
    //return {};
}));